//-----------------------------------------------------------------
// Der Inhalt der Klassen akku 
//-----------------------------------------------------------------
#include <stdio.h>
#include <iostream.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include <qlist.h>
#include <qdir.h>
#include <qregexp.h>
#include <qstring.h>
#include "akku.h"

//--------------*** Klasse akku_model ***-----------------------------------------------
//-- Konstruktor, Inhalt initialisieren

akku_model::akku_model()
{
//  printf("Messreihe wird initialisiert\n");
  messreihe.setAutoDelete(TRUE);	// Item loeschen, wenn Zeiger geloescht wird
  messreihe.clear();
  aktuell = NULL;
}

akku_model::~akku_model()
{
//  printf("Objekt Akku loeschen\n"); 
}

void akku_model::messwert_anhaengen( messwert *m )
{
//  printf("Messwert anhaengen zeit=%d\n",m->get_zeit());
//  messreihe.inSort(m);
  if(m->get_modus() != messwert::start && m->get_modus() != messwert::stop)
  {
    if(status == messwert::stop)	// wenn das letzte die Stop Meldung war
       messreihe.clear();               // vor anfang der neuen Reihe, alte loeschen

    messreihe.append(m);
    aktuell = m;		// den aktuellen Messwert merken
  }
  else
  {
    if(m->get_modus() == messwert::start)
   {
       aktuell = NULL;
       messreihe.clear();
       printf("Neue Messreihe anfangen\n");
    }
    if(m->get_modus() == messwert::stop)
    {
    if(status != messwert::stop)	// wenn das letzte nicht die Stop Meldung war
      {
      QDateTime dt = QDateTime::currentDateTime();
      QString timestr = dt.toString();

//printf("Zeitangabe = %s\n",(const char*)timestr);
      timestr.replace( QRegExp(" "), "-");
      timestr.replace( QRegExp(":"), "_");

      QString h = QDir::homeDirPath() + "/.lader_messreihen";
      mkdir((const char*)h,0755);
      h += "/" + timestr + ".dat";

     cout <<  "Filename = " << h;


      messreihe_sichern(h);
      printf("Die Messreihe ist beendet ... Daten sichern\n");
      }
    }
  }
  status = m->get_modus();	// merke, welchen Status das Ladegeraet meldet
  change();		// und die Aenderung im Model mitteilen
}

void akku_model::messreihe_ausgeben( void )
{
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    m->print();
}

int akku_model::max_spannung( void )
{
  int max = 0;
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    if(m->get_spannung() > max)
       max = m->get_spannung();
  return max;
}

int akku_model::min_spannung( void )
{
  int min = 1000000;
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    if(m->get_spannung() < min)
       min = m->get_spannung();
  return min;
}

int akku_model::max_strom( void )
{
  int max = 0;
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    if(m->get_strom() > max)
       max = m->get_strom();
  return max;
}

int akku_model::min_strom( void )
{
  int min = 1000000;
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    if(m->get_strom() < min)
       min = m->get_strom();
  return min;
}

int akku_model::max_zeit( void )
{
  int max = 0;
  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
    if(m->get_zeit() > max)
       max = m->get_zeit();
  return max;
}

QList_akku *akku_model::get_messreihe( void )
{
  return &messreihe;
}

messwert *akku_model::get_aktuell( void )
{
  return aktuell;
}

void akku_model::messreihe_sichern( const char *filename )
{

  FILE *fil;
  fil = fopen(filename,"w");
  if(fil)
  {
      messwert *m;
      for ( m=messreihe.first(); m != 0; m=messreihe.next() )
        m->save(fil);


      fclose(fil);
  }
}

void akku_model::messreihe_laden( const char *filename )
{
  FILE *fil;
  fil = fopen(filename,"r");
  if(fil)
  {
     messreihe.clear();
     int zeit,strom,spannung;
     while(fscanf(fil,"%d:%d:%d",&zeit,&strom,&spannung) > 1)
     {
      messwert *m = new messwert(zeit,strom,spannung);
      messreihe.append(m);	// nur anhaengen, aktuelle Werte nicht veraendern
      aktuell=m;		// letzter messwert als "aktuell"
     }
      change();			// dem Modell mitteilen, dass sich was gaendert hat
      fclose(fil);
  }
}

// Feststellen, ob das File eine Messreihe sein koennte
//------------------------------------------------------------------------------------
bool akku_model::messreihe_testen( const char *filename )
{
  FILE *fil;
  int dummy;

  fil = fopen(filename,"r");
  if(fil)
  {		// wenn 3 Werte mit Doppelpunkt getrennt erscheinen -> wahrscheinlich OK
     if(fscanf(fil,"%d:%d:%d",&dummy,&dummy,&dummy) >= 3)
     {
      fclose(fil);
      return TRUE;       
     }
      fclose(fil);
  }
return FALSE;
}


// errechnet die Kapazitaet aus den strom Spannungs und Zeitwerten
//----------------------------------------------------------------
double akku_model::kapazitaet( void )
{
  int last_time = 0;
  float kap = 0.0;		// kapazitaet in Ah

  messwert *m;
  for ( m=messreihe.first(); m != 0; m=messreihe.next() )
  {
    kap += (m->get_zeit() - last_time)*(m->get_strom()/1000.0 /3600.0) ;
    last_time = m->get_zeit();
  }
  return kap;
}

